//User must modify the below package with their package name
package com.SPISlaveDemo; 
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import android.app.Activity;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbAccessory;
import android.hardware.usb.UsbManager;
import android.os.Handler;
import android.os.Message;
import android.os.ParcelFileDescriptor;
import android.util.Log;
import android.widget.Toast;


/******************************CH343 SPI Slave interface class******************************************/
public class WCHSPISlaveInterface extends Activity
{
	public boolean senddata_first_flag = true;
	//define Endpoint 'vMaxPacketSize',depending on CH343 USB host
	private static final byte  IN_EP = 64;
	private static final byte OUT_EP = 64;
	private int numReadBytes_int = 0;
	public boolean dataOrder_dir = true; //set true: MSB, set false: LSB
	
	private static final String ACTION_USB_PERMISSION =    "com.SPISlaveDemo.USB_PERMISSION";
	public UsbManager usbmanager;
	public UsbAccessory usbaccessory;
	public PendingIntent mPermissionIntent;
	public ParcelFileDescriptor filedescriptor;
	public FileInputStream inputstream;
	public FileOutputStream outputstream;
	public boolean mPermissionRequestPending = false;
	public boolean READ_ENABLE = true;
	public boolean accessory_attached = false;
	public boolean terminate_read = false;
	public handler_thread handlerThread;
	
	private byte [] usbdata;
	private byte []	writeusbdata;
	private byte [] readBuffer;
	private int totalRxBytes;
	private int writeIndex;
	private int readIndex;
    private int readcount;
    private byte status;
    private int  maxnumbytes = 0xff; /*maximum data bytes, except command*/
    private int maxtxmitsize = 0xff; /*this could be max, 255*/
    public boolean datareceived = false;
    private byte IN_maxnumbytes = (IN_EP - 2); /* maximum data bytes, except command, cmd:len:data */
	private byte OUT_maxnumbytes = (OUT_EP - 2);
	
    public Context global_context;
   
    public static String ManufacturerString = "mManufacturer=WCH";
    public static String ModelString = "mModel=WCHSPISlaveDemo";
    public static String VersionString = "mVersion=1.0";	
		
		/*constructor*/
	 public WCHSPISlaveInterface(Context context){
		 	super();
            global_context = context;
			/*shall we start a thread here or what*/
			usbdata = new byte[maxtxmitsize];
			/*
			 * this shoudl be higher buffer,as this is the main
			 * 	receive buffer. But here its only 128 bytes 
			 */
			readBuffer = new byte [maxnumbytes];
			
	        writeusbdata = new byte[maxtxmitsize];
	        
	        readIndex = 0;
	        writeIndex = 0;
	        totalRxBytes = 0;
	        
	        /***********************USB handling******************************************/
			
	        usbmanager = (UsbManager) context.getSystemService(Context.USB_SERVICE);
	       // Log.d("LED", "usbmanager" +usbmanager);
	        mPermissionIntent = PendingIntent.getBroadcast(context, 0, new Intent(ACTION_USB_PERMISSION), 0);
	        IntentFilter filter = new IntentFilter(ACTION_USB_PERMISSION);
	       filter.addAction(UsbManager.ACTION_USB_ACCESSORY_DETACHED);
	       context.registerReceiver(mUsbReceiver, filter);
	        
	       inputstream = null;
	       outputstream = null;
		}
	 
	 	/*reset method*/
	 	public void Reset()
	 	{
	 		dataOrder_dir = true;
	 		
	 		/*create the packet*/
	 		writeusbdata[0] = (byte) 0x53;
	 		writeusbdata[1] = 0x00;
	 		writeusbdata[2] = 0x00;
	 		writeusbdata[3] = 0x00;
	 		writeusbdata[4] = 0x00;
			/*send the packet over the USB*/
			SendPacket((int)5);
	 	}
	 	
	 	
	 	public void SetConfig(byte clockPhase,byte dataOrder)
	 	{
	 		byte tmp = 0x00;
	 		
	 		//CMD byte
	 		writeusbdata[0] = (byte)0x50;
	 		
	 		//Master or Slave, bit 7 to represent
	 		tmp |= (1 << 7); //set 1: Slave
	 		
	 		//clockPhase, bit 6 to represent
	 		switch(clockPhase)
	 		{
	 			case 0: tmp &= ~(1 << 6);break;		//CPOL=0, CPHA=0 (mode 0)
	 			case 1:	tmp |= (1 << 6);break;		//CPOL=1, CPHA=1 (mode 3)
	 			default: tmp |= (1 << 6);break;		//CPOL=1, CPHA=1 (mode 3)
	 		}
	 		//dataOrder, bit 3,4 to reserve
	 		switch(dataOrder)
	 		{
	 			case 0: dataOrder_dir = true;break; //MSB
	 			case 1: dataOrder_dir = false;break; //LSB
	 			default: dataOrder_dir = true;break; //default MSB
	 		}
	 		
	 		writeusbdata[1] = tmp;
	 		
	 		writeusbdata[2] = 0x00;//to reserve
	 		writeusbdata[3] = 0x00;
	 		writeusbdata[4] = 0x00;
	 		
	 		SendPacket((int)5);;
	 		
	 	}
	 		
	 	
	 	/*write data*/
	 	public byte SendData(int numBytes, byte [] buffer, byte [] numReadBytes)
	 	{
	 		byte send_len = 0;
			int send_recv = 0;
			int send_total = 0;
			byte[] buffer_tmp = new byte[255];
			
			int dataOrder_count = 0;
			byte tmp      = 0x00;
			int  i = 0;
			
			status = 0x00; /* error by default */
			/*
			 * if num bytes are more than maximum limit
			 */
			if (numBytes < 1) {
				/* return the status with the error in the command */
				return status;
			}
			

			if( dataOrder_dir == true ) // MSB mode
			{
				for(dataOrder_count = 0; dataOrder_count < numBytes; dataOrder_count++)
				{
					buffer_tmp[dataOrder_count] = buffer[dataOrder_count];
				}
			}
			else if( dataOrder_dir == false) //handle LSB mode
			{
				for(dataOrder_count = 0; dataOrder_count < numBytes; dataOrder_count++)
				{
					tmp = 0;
					for( i = 0; i < 8; i ++)
					{
						if( ( buffer[dataOrder_count] & (1 << i) ) != 0)
						{
							tmp |= ( 1 << (7 - i) );
						}
						
					}
					buffer_tmp[dataOrder_count] = tmp;
				}
			}
			
			while( numBytes > 0 ) 
			{
				// first packet
				if ( senddata_first_flag == true )
				{
					senddata_first_flag = false;
					writeusbdata[0] = (byte) 0x51;
					writeusbdata[1] = (byte)(numBytes & 0xff);
				
					if( numBytes <= IN_maxnumbytes ) //numBytes <= 62
					{
						send_len = (byte)(numBytes & 0xff);
						numBytes = 0;
					}
					else if( numBytes > IN_maxnumbytes ) //numBytes > 62
					{
						send_len = IN_maxnumbytes;
						numBytes -= IN_maxnumbytes;
					}
					for(int count = 0; count < send_len; count++ ){
						writeusbdata[count + 2] = (byte)buffer_tmp[send_total];
						send_total++;
					}
					SendPacket((int) (send_len + 2));
					continue;
				}
				//next packet
				if ( numBytes >= (IN_maxnumbytes + 2) )
				{
					send_len = (byte)(IN_maxnumbytes + 2);
					numBytes -= (IN_maxnumbytes + 2);
					
					for( int count = 0; count < send_len; count++ ){
						writeusbdata[count] = (byte)buffer_tmp[send_total];
						send_total++;
					}
					SendPacket((int) send_len);
					continue;
				}
				else if ( numBytes < (IN_maxnumbytes + 2)) {
					send_len = (byte)(numBytes & 0xff);
					numBytes = 0;
					
					for( int count = 0; count < send_len; count++ ){
						writeusbdata[count] = (byte)buffer_tmp[send_total];
						send_total++;
					}
					SendPacket((int) send_len);
					continue;
				}
			}
			
			numReadBytes[0] = (byte)(numBytes & 0xff);
			senddata_first_flag = true; // Ready for next SendPacket;
			
			return status;
	 	}
	 	
	 	private void receive_reply()
		{
			datareceived = false;
			while(true) 
			{
				if (datareceived == true) {
					break;
				}
			}
		}
	 	
	 	/*read data*/
	 	public byte ReadData(byte numBytes,byte[] buffer, byte [] actualNumBytes)
	 	{
	 		int dataOrder_count = 0;
	 		byte tmp      = 0x00;
			int  i = 0;
	 		
	 		status = 0x00; /*success by default*/
	 			
	 		/*should be at least one byte to read*/
	 		if((numBytes < 1) || (totalRxBytes == 0)){
	 			actualNumBytes[0] = 0x00;
	 			status = 0x01;
	 			return status;
	 		}
	 			
	 		/*check for max limit*/
	 		if(numBytes > maxtxmitsize){
	 			numBytes = (byte)(maxtxmitsize & 0xff);
	 		}
	 			
	 			
	 		if(numBytes > totalRxBytes)
	 			numBytes = (byte)(totalRxBytes & 0xff);
	 			
	 					
	 			
	 		/*update the number of bytes available*/
	 		totalRxBytes -= numBytes;
	 				
	 		actualNumBytes[0] = numBytes;	
	 				
	 		/*copy to the user buffer*/	
 			for(int count = 0; count<numBytes;count++)
 			{
 				buffer[count] = readBuffer[readIndex];
 				readIndex++;
 				/*shouldnt read more than what is there in the buffer,
 				* so no need to check the overflow
 				*/
 				readIndex %= maxnumbytes;
 			}
 				
 			if( dataOrder_dir == true ) // MSB mode
 			{
 				//Nothing to do
 			}
 			else if( dataOrder_dir == false) //handle LSB mode
 			{
 				for(dataOrder_count = 0; dataOrder_count < numBytes; dataOrder_count++)
 				{
 					tmp = 0;
 					for( i = 0; i < 8; i ++)
 					{
 						if( ( buffer[dataOrder_count] & (1 << i) ) != 0)
 						{
 							tmp |= ( 1 << (7 - i) );
 						}
 					}
 					buffer[dataOrder_count] = tmp;
 				}
 			}
 				
 				return status;
	 	}
	 		 	
		/*method to send on USB*/
		private void SendPacket(int numBytes)
		{
			
			try {
				if(outputstream != null){
					outputstream.write(writeusbdata, 0,numBytes);
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}
		
		/*resume accessory*/
		public void ResumeAccessory()
		{
			// Intent intent = getIntent();
			if (inputstream != null && outputstream != null) {
				return;
			}
			
			UsbAccessory[] accessories = usbmanager.getAccessoryList();
			if(accessories != null)
			{
				Toast.makeText(global_context, "Accessory Attached", Toast.LENGTH_SHORT).show();
			}		
			else
			{
				accessory_attached = false;
				return;
			}
			
			UsbAccessory accessory = (accessories == null ? null : accessories[0]);
			if (accessory != null) {
				if( -1 == accessory.toString().indexOf(ManufacturerString))
				{
					Toast.makeText(global_context, "Manufacturer is not matched!", Toast.LENGTH_SHORT).show();
					return;
				}

				if( -1 == accessory.toString().indexOf(ModelString))
				{
					Toast.makeText(global_context, "Model is not matched!", Toast.LENGTH_SHORT).show();
					return;
				}
				
				if( -1 == accessory.toString().indexOf(VersionString))
				{
					Toast.makeText(global_context, "Version is not matched!", Toast.LENGTH_SHORT).show();
					return;
				}
				
				Toast.makeText(global_context, "Manufacturer, Model & Version are matched!", Toast.LENGTH_SHORT).show();
				accessory_attached = true;
				
				if (usbmanager.hasPermission(accessory)) {
					OpenAccessory(accessory);
				} 
				else
				{
					synchronized (mUsbReceiver) {
						if (!mPermissionRequestPending) {
							Toast.makeText(global_context, "Request USB Permission", Toast.LENGTH_SHORT).show();
							usbmanager.requestPermission(accessory,
									mPermissionIntent);
							mPermissionRequestPending = true;
						}
				}
			}
			} else {}

		}
		
		/*destroy accessory*/
		public void DestroyAccessory(){
			global_context.unregisterReceiver(mUsbReceiver);
			CloseAccessory();
		}
		
		public void EndHandleThread()
		{
			READ_ENABLE = false;
			terminate_read = true;
		}
		
/*********************helper routines*************************************************/		
		
		public void OpenAccessory(UsbAccessory accessory)
		{
			filedescriptor = usbmanager.openAccessory(accessory);
			if(filedescriptor != null){
				usbaccessory = accessory;
				FileDescriptor fd = filedescriptor.getFileDescriptor();
				inputstream = new FileInputStream(fd);
				outputstream = new FileOutputStream(fd);
				/*check if any of them are null*/
				if(inputstream == null || outputstream==null){
					return;
				}
			}
			
			handlerThread = new handler_thread(inputstream);
			handlerThread.start();
		}
		
		private void CloseAccessory()
		{
			try{
				if(filedescriptor != null)
					filedescriptor.close();
				
			}catch (IOException e){}
			
			try {
				if(inputstream != null)
						inputstream.close();
			} catch(IOException e){}
			
			try {
				if(outputstream != null)
						outputstream.close();
				
			}catch(IOException e){}
			/*FIXME, add the notfication also to close the application*/
			
			filedescriptor = null;
			inputstream = null;
			outputstream = null;
		
			System.exit(0);
		
		}
		
				
		/***********USB broadcast receiver*******************************************/
	    private final BroadcastReceiver mUsbReceiver = new BroadcastReceiver() 
		{
			@Override
			public void onReceive(Context context, Intent intent) 
			{
				String action = intent.getAction();
				if (ACTION_USB_PERMISSION.equals(action)) 
				{	
					synchronized (this)
					{
						UsbAccessory accessory = (UsbAccessory) intent.getParcelableExtra(UsbManager.EXTRA_ACCESSORY);
						if (intent.getBooleanExtra(UsbManager.EXTRA_PERMISSION_GRANTED, false))
						{
							Toast.makeText(global_context, "Allow USB Permission", Toast.LENGTH_SHORT).show();
							OpenAccessory(accessory);
						} 
						else 
						{
							Toast.makeText(global_context, "Deny USB Permission", Toast.LENGTH_SHORT).show();
							Log.d("LED", "permission denied for accessory "+ accessory);
							
						}
						mPermissionRequestPending = false;
					}
				} 
				else if (UsbManager.ACTION_USB_ACCESSORY_DETACHED.equals(action)) 
				{
						CloseAccessory();
				}else
				{
					Log.d("LED", "....");
				}
			}	
		};
	    

		
		/*usb input data handler*/
		private class handler_thread  extends Thread {
			FileInputStream instream;
			
			handler_thread(FileInputStream stream ){
				instream = stream;
			}
			
			public void run()
			{
				
				while(READ_ENABLE == true)
				{
					
					try
					{
						/*dont overwrite the previous buffer*/
						if((instream != null) && (datareceived == false))
						{
							readcount = instream.read(usbdata,0,64);
							if(readcount > 0)
							{
								datareceived = true;
								
								for( int count = 0; count < readcount; count++ )
								{
									readBuffer[writeIndex] = usbdata[count];
									/* move to the next write location */
									writeIndex++;
									writeIndex %= maxnumbytes;
									/* FIXME, check for overflow */
									
									if( writeIndex >= readIndex )
										totalRxBytes = writeIndex - readIndex;
									else
										totalRxBytes = (maxnumbytes - readIndex ) + writeIndex;
									
									datareceived = false;
								}
							}
						
						}
					}catch (IOException e){}
				}
				
				if(terminate_read == true)
				{
				        Log.e(">>@@","terminate handler_thread");
					DestroyAccessory();
				}
			}
		}
	}